@@ -17,11 +17,13 @@
 
 import gtk, os, pango, gquilt_utils, time
 
-def _read_fs_entity_name(flag, prompt, startdir, ok_button_label):
+def _read_fs_entity_name(flag, prompt, startdir, ok_button_label, sugg=""):
     dialog = gtk.FileChooserDialog(prompt, None, flag,
         (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, ok_button_label, gtk.RESPONSE_OK))
     dialog.set_default_response(gtk.RESPONSE_OK)
     dialog.set_position(gtk.WIN_POS_MOUSE)
+    if flag == gtk.FILE_CHOOSER_ACTION_SAVE:
+        dialog.set_current_name(sugg)
     if startdir != None:
         dialog.set_current_folder(startdir)
     response = dialog.run()
@@ -29,11 +31,13 @@ def _read_fs_entity_name(flag, prompt, startdir, ok_button_label):
         res = (True, dialog.get_filename())
     elif response == gtk.RESPONSE_CANCEL:
         res = (False, None)
+    else:
+        res = (True, os.getcwd())
     dialog.destroy()
     return res
 
-def read_new_file_name(prompt="New file ...", startdir=None, ok_button_label=gtk.STOCK_OPEN):
-    return _read_fs_entity_name(gtk.FILE_CHOOSER_ACTION_SAVE, prompt, startdir, ok_button_label)
+def read_new_file_name(prompt="New file ...", startdir=None, ok_button_label=gtk.STOCK_OK, sugg=""):
+    return _read_fs_entity_name(gtk.FILE_CHOOSER_ACTION_SAVE, prompt, startdir, ok_button_label, sugg)
 
 def read_file_name(prompt="File ...", startdir=None, ok_button_label=gtk.STOCK_OPEN):
     return _read_fs_entity_name(gtk.FILE_CHOOSER_ACTION_OPEN, prompt, startdir, ok_button_label)
@@ -81,6 +85,7 @@ class info_dialog(_info_dialog):
 
 FORCE = 1
 REFRESH_AND_RETRY = 2
+SKIP = 3
 CANCEL = gtk.RESPONSE_CANCEL
 
 class info_force_dialog(_info_dialog):
@@ -96,6 +101,20 @@ class info_force_dialog(_info_dialog):
         self.destroy()
         return res == FORCE
 
+class info_force_or_skip_dialog(_info_dialog):
+    def __init__(self, msg=None, parent=None):
+        _info_dialog.__init__(self, "FYI", parent,
+                              gtk.DIALOG_DESTROY_WITH_PARENT,
+                              (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                               "_Skip", SKIP,
+                               "_Force", FORCE))
+        if msg != None:
+            self.set_message(msg)
+    def ask(self):
+        res = self.run()
+        self.destroy()
+        return res
+
 class info_refresh_dialog(_info_dialog):
     def __init__(self, msg=None, parent=None):
         _info_dialog.__init__(self, "FYI", parent,
@@ -136,6 +155,19 @@ class info_ok_dialog(_info_dialog):
         self.destroy()
         return res == gtk.RESPONSE_OK
 
+class info_yes_no_dialog(_info_dialog):
+    def __init__(self, msg=None, parent=None):
+        _info_dialog.__init__(self, "FYI", parent,
+                              gtk.DIALOG_DESTROY_WITH_PARENT,
+                              (gtk.STOCK_NO, gtk.RESPONSE_NO,
+                               gtk.STOCK_YES, gtk.RESPONSE_YES))
+        if msg != None:
+            self.set_message(msg)
+    def ask(self):
+        res = self.run()
+        self.destroy()
+        return res == gtk.RESPONSE_YES
+
 class text_entry_dialog(gtk.Dialog):
     def __init__(self, prompt="Enter text:", parent=None):
         gtk.Dialog.__init__(self, "Text Entry", parent,
@@ -170,6 +202,28 @@ class text_entry_dialog(gtk.Dialog):
     def set_text(self, text):
         self.te.set_text(text)
 
+class select_from_list_dialog(gtk.Dialog):
+    def __init__(self, list=[], title="Select from list", parent=None):
+        gtk.Dialog.__init__(self, title, parent,
+                            gtk.DIALOG_DESTROY_WITH_PARENT,
+                            (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                             gtk.STOCK_OK, gtk.RESPONSE_OK)
+                           )
+        if parent is None:
+            self.set_position(gtk.WIN_POS_MOUSE)
+        else:
+            self.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
+        self.cbox = gtk.combo_box_new_text()
+        for i in list:
+            self.cbox.append_text(i)
+        self.vbox.add(self.cbox)
+        self.cbox.show()
+    def make_selection(self):
+        res = self.run()
+        str = self.cbox.get_active_text()
+        self.destroy()
+        return (res == gtk.RESPONSE_OK and str is not None, str)
+
 class update_dialog(gtk.Dialog):
     def __init__(self, title):
         gtk.Dialog.__init__(self, title, None,
@@ -308,9 +362,13 @@ class scrolled_treeview_with_popup(gtk.Frame):
         self._conditional_mi_list = []
     def set_tooltips(self, tooltips):
         self.tooltips = tooltips
+    def _popup_ok(self):
+        return True
     def _handle_button_press(self, widget, event):
         if event.type == gtk.gdk.BUTTON_PRESS:
             if event.button == 3:
+                if not self._popup_ok():
+                    return False
                 sel_sz = self.view.get_selection().count_selected_rows()
                 unique = sel_sz == 1
                 selmade = sel_sz > 0
@@ -355,6 +413,12 @@ class scrolled_treeview_with_popup(gtk.Frame):
     def append_conditional_menu_item(self, label, tooltip, stock_id, action, cond, arg=None, condarg=None):
         mi = self.append_menu_item(label, tooltip, stock_id, action, arg)
         self._conditional_mi_list.append((mi, cond, condarg))
+    def _show_busy(self):
+        self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
+        while gtk.events_pending():
+            gtk.main_iteration()
+    def _unshow_busy(self):
+        self.window.set_cursor(None)
 
 class file_tree(scrolled_treeview_with_popup):
     def __init__(self, view_title="Files", tooltips=None, frame_title=None):
@@ -367,6 +431,9 @@ class file_tree(scrolled_treeview_with_popup):
         tvcolumn.set_attributes(text_cell, text=0, style=2, foreground=3)
         tvcolumn.set_attributes(icon_cell, stock_id=4)
         self.view.append_column(tvcolumn)
+        self.console = None
+    def set_console(self, console):
+        self.console = console
     def _get_selected_files_cb(self, store, path, iter, list):
         list.append(store.fs_path(iter))
     def get_selected_files(self):
@@ -379,6 +446,8 @@ class dir_file_tree(file_tree):
         store = view.get_model()
         ci = store.iter_children(iter)
         if store.get_value(ci, 0) is None:
+            # This was the place holder row put in unpopulated directories
+            # to make them display properly so we need to remove it
             store.remove(ci)
             self._populate(self.store.fs_path(iter), iter)
         else:
