@@ -17,7 +17,7 @@
 
 # This file provides access to "quilt" functionality required by "gquilt"
 
-import gquilt_utils, os.path, os, gquilt_tool, gquilt_pfuns, re
+import gquilt_utils, os.path, os, gquilt_const, gquilt_tool, gquilt_pfuns, re, errno
 
 class quilt_commands(gquilt_utils.cmd_list):
     def __init__(self):
@@ -62,14 +62,14 @@ def internal(cmd, input=None):
     return gquilt_utils.run_cmd(" ".join([qbsfe, cmd]), input)
 
 # run a command and log the result to the provided console
-def _exec_console_cmd(console, cmd, error=gquilt_tool.ERROR):
+def _exec_console_cmd(console, cmd, error=gquilt_const.ERROR):
     res, so, se = gquilt_utils.run_cmd(cmd)
     if console is not None:
         console.log_cmd(cmd + "\n", so, se)
     if res != 0:
         return (error, so, se)
     else:
-        return (gquilt_tool.OK, so, se)
+        return (gquilt_const.OK, so, se)
 
 # Some useful private quilt functions (that shouldn't fail)
 def _patch_file_name(patch):
@@ -97,25 +97,31 @@ def _patch_files_may_have_changed(patch):
     return res == 0
 
 def _convert_pop_res(res, so, se):
-    if res != gquilt_tool.OK:
+    if res != gquilt_const.OK:
         if re.search("needs to be refreshed first", so + se):
-            res = gquilt_tool.ERROR_REFRESH
+            res = gquilt_const.ERROR_REFRESH
         elif re.search("\(refresh it or enforce with -f\)", so + se):
-            res = gquilt_tool.ERROR_FORCE_OR_REFRESH
+            res = gquilt_const.ERROR_FORCE_OR_REFRESH
         else:
-            res = gquilt_tool.ERROR
+            res = gquilt_const.ERROR
     return (res, so, se)
 
 def _convert_push_res(res, so, se):
-    if res != gquilt_tool.OK:
+    if res != gquilt_const.OK:
         if re.search("\(forced; needs refresh\)", so + se):
-            res = gquilt_tool.OK
+            res = gquilt_const.OK
         elif re.search("needs to be refreshed first", so + se):
-            res = gquilt_tool.ERROR_REFRESH
+            res = gquilt_const.ERROR_REFRESH
         elif re.search("\(enforce with -f\)", so + se):
-            res = gquilt_tool.ERROR_FORCE
+            res = gquilt_const.ERROR_FORCE
         else:
-            res = gquilt_tool.ERROR
+            res = gquilt_const.ERROR
+    return (res, so, se)
+
+def _convert_import_res(res, so, se):
+    if res != gquilt_const.OK:
+        if re.search("Replace with -f", so + se):
+            res = gquilt_const.ERROR_FORCE
     return (res, so, se)
 
 def _quilt_version_ge(rmajor, rminor):
@@ -131,13 +137,35 @@ def _quilt_version_ge(rmajor, rminor):
     else:
         return int(vstrs[1]) >= rminor
 
+def _patch_dir(dir=None):
+    try:
+        pd = os.environ['QUILT_PATCHES']
+    except:
+        pd = "patches"
+    if dir is None:
+        dir = os.getcwd()
+    return dir + "/" + pd
+    
 # Now implement the tool interface for quilt
 class interface(gquilt_tool.interface):
     def __init__(self):
         gquilt_tool.interface.__init__(self, "quilt")
+    def new_playground(self, dir=None):
+        patch_dir = _patch_dir(dir)
+        if not os.path.exists(patch_dir):
+            try:
+                os.makedirs(patch_dir)
+            except os.error, value:
+               return (gquilt_const.ERROR, "", value[1])
+        return gquilt_utils.run_cmd("touch " + os.path.join(patch_dir, "series"))
+    def is_playground(self, dir=None):
+        patch_dir = _patch_dir(dir)
+        return os.path.exists(os.path.join(patch_dir, "series"))
     def is_patch_applied(self, patch):
         res, so, se = internal("is_applied " + patch)
         return res == 0
+    def patch_file_name(self, patch):
+        return _patch_file_name(patch)
     def top_patch(self):
         res, so, se = gquilt_utils.run_cmd("quilt top")
         if res == 0 or (se.strip() == "" and so.strip() == ""):
@@ -151,26 +179,17 @@ class interface(gquilt_tool.interface):
         if res != 0:
             raise "quilt_specific_error", se
         return op.splitlines()[-1]
-    def display_file_diff_in_viewer(self, viewer, file, patch=None):
+    def count_ok_meld(selfself, count):
+        return count == 1
+    def display_files_diff_in_viewer(self, viewer, files, patch=None):
         difffld = "--diff=" + viewer
         if patch is None:
-            patchfld = ""
+            pid = os.spawnlp(os.P_NOWAIT, "quilt", "quilt", "diff", difffld, files[0])
         else:
-            patchfld = "-P " + patch
-        pid = os.spawnlp(os.P_NOWAIT, "quilt", "quilt", "diff", difffld, patchfld, file)
-    def get_patch_description(self, patch):
-        pfn = _patch_file_name(patch)
-        if os.path.exists(pfn):
-            res, lines = gquilt_pfuns.get_patch_descr_lines(pfn)
-            if res:
-                return (gquilt_tool.OK, "\n".join(lines) + "\n", "")
-            else:
-                return (gquilt_tool.ERROR, "", "Error reading patch description\n")
-        else:
-            return (gquilt_tool.OK, "", "")
+            pid = os.spawnlp(os.P_NOWAIT, "quilt", "quilt", "diff", difffld, "-P", patch, files[0])
     def get_patch_status(self, patch):
         if not self.is_patch_applied(patch):
-            return (gquilt_tool.OK, gquilt_tool.NOT_APPLIED, "")
+            return (gquilt_const.OK, gquilt_const.NOT_APPLIED, "")
         patch_fn = _patch_file_name(patch)
         patch_dirn = _patch_backup_dir(patch)
         if (not os.path.exists(patch_fn)) or os.path.exists(patch_dirn + "~refresh"):
@@ -193,32 +212,32 @@ class interface(gquilt_tool.interface):
         res, tp, se = self.get_top_patch()
         if tp == patch:
             if needs_refresh:
-                return (gquilt_tool.OK, gquilt_tool.TOP_PATCH_NEEDS_REFRESH, "")
+                return (gquilt_const.OK, gquilt_const.TOP_PATCH_NEEDS_REFRESH, "")
             else:
-                return (gquilt_tool.OK, gquilt_tool.TOP_PATCH, "")
+                return (gquilt_const.OK, gquilt_const.TOP_PATCH, "")
         else:
             if needs_refresh:
-                return (gquilt_tool.OK, gquilt_tool.APPLIED_NEEDS_REFRESH, "")
+                return (gquilt_const.OK, gquilt_const.APPLIED_NEEDS_REFRESH, "")
             else:
-                return (gquilt_tool.OK, gquilt_tool.APPLIED, "")
+                return (gquilt_const.OK, gquilt_const.APPLIED, "")
     def get_series(self):
         res, op, err = gquilt_utils.run_cmd("quilt series -v")
         if res != 0:
-            return (gquilt_tool.ERROR, (None, None), err)
+            return (gquilt_const.ERROR, (None, None), err)
         series = []
         for line in op.splitlines():
             if line[0] == " ":
-                series.append((line[2:], gquilt_tool.NOT_APPLIED))
+                series.append((line[2:], gquilt_const.NOT_APPLIED))
             else:
                 pn = line[2:]
                 #res, status, err = self.get_patch_status(pn)
                 #series.append((pn, status))
-                series.append((pn, gquilt_tool.APPLIED))
+                series.append((pn, gquilt_const.APPLIED))
         try:
             patch_dir = os.environ['QUILT_PATCHES']
         except:
             patch_dir = ""
-        return (gquilt_tool.OK, (patch_dir, series), "")
+        return (gquilt_const.OK, (patch_dir, series), "")
     def get_diff(self, filelist=[], patch=None):
         if patch is None:
             cmd = "quilt diff"
@@ -226,7 +245,11 @@ class interface(gquilt_tool.interface):
             cmd = "quilt diff -P " + patch
         res, so, se = gquilt_utils.run_cmd(" ".join([cmd, " ".join(filelist)]))
         if res != 0:
-            return (gquilt_tool.ERROR, so, se)
+            if not self.is_patch_applied(patch):
+                res, diff = gquilt_pfuns.get_patch_diff_lines(self.patch_file_name(patch))
+                if res:
+                    return (gquilt_const.OK, "\n".join(diff), [])
+            return (gquilt_const.ERROR, so, se)
         return (res, so, se)
     def get_combined_diff(self, start_patch=None, end_patch=None):
         cmd = "quilt diff --sort --combine "
@@ -238,11 +261,11 @@ class interface(gquilt_tool.interface):
             cms += " -P " + end_patch
         res, so, se = gquilt_utils.run_cmd(cmd)
         if res != 0:
-            return (gquilt_tool.ERROR, so, se)
+            return (gquilt_const.ERROR, so, se)
         return (res, so, se)
     def get_patch_files(self, patch=None, withstatus=True):
         if self.top_patch() == "":
-            return (gquilt_tool.OK, "", "")
+            return (gquilt_const.OK, "", "")
         cmd = "quilt files"
         if withstatus:
             cmd += " -v"
@@ -250,31 +273,26 @@ class interface(gquilt_tool.interface):
             cmd += " " + patch
         res, so, se = gquilt_utils.run_cmd(cmd)
         if res != 0:
-            return (gquilt_tool.ERROR, so, se)
+            if patch and not self.is_patch_applied(patch):
+                patchfile = self.patch_file_name(patch)
+                ok, filelist = gquilt_pfuns.get_patch_files(patchfile, withstatus)
+                if ok:
+                    return (gquilt_const.OK, filelist, "")
+                else:
+                    return (gquilt_const.ERROR, "", filelist)
+            return (gquilt_const.ERROR, so, se)
         if withstatus:
             filelist = []
             for line in so.splitlines():
                 if line[0] == "+":
-                    filelist.append((line[2:], gquilt_tool.ADDED))
+                    filelist.append((line[2:], gquilt_const.ADDED))
                 elif line[0] == "-":
-                    filelist.append((line[2:], gquilt_tool.DELETED))
+                    filelist.append((line[2:], gquilt_const.DELETED))
                 else:
-                    filelist.append((line[2:], gquilt_tool.EXTANT))
+                    filelist.append((line[2:], gquilt_const.EXTANT))
         else:
             filelist = so.splitlines()
         return (res, filelist, se)
-    def do_set_patch_description(self, console, patch, description):
-        pfn = _patch_file_name(patch)
-        res = gquilt_pfuns.set_patch_descr_lines(pfn, description.splitlines())
-        if res:
-            res = gquilt_tool.OK
-            se = ""
-        else:
-            res = gquilt_tool.ERROR
-            se = "Error reading patch description\n"
-        if console is not None:
-            console.log_cmd('set description for "' + patch + '"', "\n", se)
-        return (res, "", se)
     def do_rename_patch(self, console, patch, newname):
         if quilt_commands().has_cmd("rename"):
             cmd = "quilt rename "
@@ -288,23 +306,23 @@ class interface(gquilt_tool.interface):
             if console is not None:
                 console.log_cmd(cmd + "\n", so, se)
             if res != 0:
-                return (gquilt_tool.ERROR, so, se)
+                return (gquilt_const.ERROR, so, se)
             else:
-                return (gquilt_tool.OK, so, se)
+                return (gquilt_const.OK, so, se)
         else:
             newname_pfn = _patch_file_name(newname)
             newname_backup_dir =  _patch_backup_dir(newname)
             if os.path.exists(newname_pfn) or os.path.exists(newname_backup_dir):
-                return (gquilt_tool.ERROR, "", 'Patch rename failed. "' + newname + '" is already being used')
+                return (gquilt_const.ERROR, "", 'Patch rename failed. "' + newname + '" is already being used')
             if patch is None:
                 res, patch, se = self.get_top_patch()
-                if res != gquilt_tool.OK:
+                if res != gquilt_const.OK:
                     return (res, patch, se)
             patchname_pfn = _patch_file_name(patch)
             if self.is_patch_applied(patch):
                 res, so, se = internal(" ".join(["rename_in_db",  patch, newname]))
                 if res != 0:
-                    return (gquilt_tool.ERROR, so, se)
+                    return (gquilt_const.ERROR, so, se)
                 os.rename(_patch_backup_dir(patch), newname_backup_dir)
                 if os.path.exists(patchname_pfn):
                     os.rename(patchname_pfn, newname_pfn)
@@ -313,9 +331,9 @@ class interface(gquilt_tool.interface):
                     os.rename(patchname_pfn, newname_pfn)
             res, so, se = internal(" ".join(["rename_in_series", patch, newname]))
             if res != 0:
-                return (gquilt_tool.ERROR, so, se)
+                return (gquilt_const.ERROR, so, se)
             else:
-                return (gquilt_tool.OK, so, se)
+                return (gquilt_const.OK, so, se)
 
     def do_pop_patch(self, console, force=False):
         cmd = "quilt pop"
@@ -354,17 +372,30 @@ class interface(gquilt_tool.interface):
         if patch is not None:
             cmd = " ".join([cmd, patch])
         res, so, se = _exec_console_cmd(console, cmd)
-        if res != gquilt_tool.OK:
+        if res != gquilt_const.OK:
             if re.search("Enforce refresh with -f", so + se):
-                res = gquilt_tool.ERROR_FORCE
+                res = gquilt_const.ERROR_FORCE
             else:
-                res = gquilt_tool.ERROR
+                res = gquilt_const.ERROR
         return (res, so, se)
     def do_create_new_patch(self, console, name):
         return _exec_console_cmd(console, " ".join(["quilt", "new", name]))
-    def do_import_patch(self, console, filename):
-        return _exec_console_cmd(console, " ".join(["quilt", "import", filename]))
-    def do_merge_patch(self, console, filename):
+    def do_import_patch(self, console, filename, patchname=None, force=False):
+        if patchname:
+            cmd = "quilt import -P %s " % patchname
+        else:
+            cmd = "quilt import "
+        if force:
+            cmd += "-f -d n "
+        res, so, se = _exec_console_cmd(console, cmd + filename)
+        return _convert_import_res(res, so, se)
+    def do_merge_patch(self, console, patch):
+        filename = _patch_file_name(patch)
+        res, so, se = self.do_merge_patch_file(console, filename)
+        if res == 0:
+            self.do_delete_patch(console, patch)
+        return (res, so, se)
+    def do_merge_patch_file(self, console, filename):
         return _exec_console_cmd(console, " < ".join(["quilt fold", filename]))
     def do_delete_patch(self, console, patch):
         return _exec_console_cmd(console, " ".join(["quilt", "delete", patch]))
@@ -379,7 +410,7 @@ class interface(gquilt_tool.interface):
         # quilt will screw up semi silently if you try to add directories to a patch
         for f in filelist:
             if os.path.isdir(f):
-                return (gquilt_tool.ERROR, "", f + ' is a directory.\n "quilt" does not handle adding directories to patches\n')
+                return (gquilt_const.ERROR, "", f + ' is a directory.\n "quilt" does not handle adding directories to patches\n')
         return _exec_console_cmd(console, " ".join([cmd, " ".join(filelist)]))
     def do_remove_files_from_patch(self, console, filelist, patch=None):
         if patch == None:
@@ -392,7 +423,7 @@ class interface(gquilt_tool.interface):
         # quilt will screw up semi silently if you try to remove directories to a patch
         for f in filelist:
             if os.path.isdir(f):
-                return (gquilt_tool.ERROR, "", f + ' is a directory.\n "quilt" does not handle removing directories from patches\n')
+                return (gquilt_const.ERROR, "", f + ' is a directory.\n "quilt" does not handle removing directories from patches\n')
         return _exec_console_cmd(console, " ".join([cmd, " ".join(filelist)]))
     def do_exec_tool_cmd(self, console, cmd):
         return _exec_console_cmd(console, " ".join(["quilt", cmd]))
