@@ -72,6 +72,16 @@ def run_cmd(cmd=None, input=None):
 def is_hidden_file(f):
     return f[0] == '.'
 
+def path_relative_to_dir(dir, path):
+    if not os.path.isdir(dir):
+        return None
+    if dir == path:
+        return "."
+    lcwd = len(dir)
+    if len(path) <= lcwd + 1 or dir != path[0:lcwd] or path[lcwd] != "/":
+        return None
+    return path[lcwd + 1:]
+
 def dir_contents(dir):
     entries = os.listdir(dir)
     hfiles = []
